import pytest
from threatexchange.cli.tests.e2e_test_helper import (
    ThreatExchangeCLIE2eHelper,
    te_cli,
)
from threatexchange.exchanges.clients.ncmec.hash_api import NCMECEnvironment


@pytest.fixture
def cli(
    te_cli: ThreatExchangeCLIE2eHelper,
) -> ThreatExchangeCLIE2eHelper:
    te_cli.COMMON_CALL_ARGS = ("config",)
    return te_cli


def test_config_signal(cli: ThreatExchangeCLIE2eHelper) -> None:
    expected = [
        "pdq threatexchange.signal_type.pdq.signal.PdqSignal",
        "raw_text threatexchange.signal_type.raw_text.RawTextSignal",
        "trend_query threatexchange.signal_type.trend_query.TrendQuerySignal",
        "url threatexchange.signal_type.url.URLSignal",
        "url_md5 threatexchange.signal_type.url_md5.UrlMD5Signal",
        "video_md5 threatexchange.signal_type.md5.VideoMD5Signal",
    ]
    cli.assert_cli_output(["signal"], expected)
    cli.assert_cli_output(["signal", "list"], expected)


def test_config_content(cli: ThreatExchangeCLIE2eHelper) -> None:
    expected = [
        "photo threatexchange.content_type.photo.PhotoContent",
        "text threatexchange.content_type.text.TextContent",
        "url threatexchange.content_type.url.URLContent",
        "video threatexchange.content_type.video.VideoContent",
    ]
    cli.assert_cli_output(["content"], expected)
    cli.assert_cli_output(["content", "list"], expected)



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved

"""
Returns the hardcoded example signals from SignalType implementations.

This makes it easier to demonstrate new signal types locally, even without
access to any API.

The CLI defaults to this being the only collaboration.
"""


import typing as t

from threatexchange.signal_type.signal_base import SignalType

from threatexchange.exchanges import fetch_state as state
from threatexchange.exchanges.collab_config import CollaborationConfigBase
from threatexchange.exchanges.signal_exchange_api import (
    SignalExchangeAPIWithSimpleUpdates,
)

_TypedDelta = state.FetchDelta[
    t.Tuple[str, str],
    state.FetchedSignalMetadata,
    state.NoCheckpointing,
]


class StaticSampleSignalExchangeAPI(
    SignalExchangeAPIWithSimpleUpdates[
        CollaborationConfigBase,
        state.NoCheckpointing,
        state.FetchedSignalMetadata,
    ]
):
    """Return a static set of sample data for demonstration"""

    @classmethod
    def get_name(cls) -> str:
        return "sample"

    @staticmethod
    def get_config_cls() -> t.Type[CollaborationConfigBase]:
        return CollaborationConfigBase

    @staticmethod
    def get_checkpoint_cls() -> t.Type[state.NoCheckpointing]:
        return state.NoCheckpointing

    @staticmethod
    def get_record_cls() -> t.Type[state.FetchedSignalMetadata]:
        return state.FetchedSignalMetadata

    @classmethod
    def for_collab(
        cls, collab: CollaborationConfigBase
    ) -> "StaticSampleSignalExchangeAPI":
        return cls()

    def fetch_iter(
        self,
        supported_signal_types: t.Sequence[t.Type[SignalType]],
        checkpoint: t.Optional[state.TFetchCheckpoint],
    ) -> t.Iterator[_TypedDelta]:
        sample_signals: t.List[
            t.Tuple[t.Tuple[str, str], state.FetchedSignalMetadata]
        ] = []
        for stype in supported_signal_types:
            sample_signals.extend(_signals(stype))

        updates: t.Dict[
            t.Tuple[str, str], t.Optional[state.FetchedSignalMetadata]
        ] = dict(sample_signals)

        yield _TypedDelta(
            updates,
            state.NoCheckpointing(),
        )


def _signals(
    sig_cls: t.Type[SignalType],
) -> t.Iterable[t.Tuple[t.Tuple[str, str], state.FetchedSignalMetadata]]:
    sig_name = sig_cls.get_name()
    return (
        ((sig_name, s), state.FetchedSignalMetadata()) for s in sig_cls.get_examples()
    )



#!/usr/bin/env python
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved

"""
A place to put simple helpers that don't seem like they go anywhere else.

If this file starts getting large, break it up.
"""

import argparse
import typing as t
import re
from urllib.parse import urlparse
import unicodedata


def class_name_to_human_name(name: str, suffix: str) -> str:
    """Helper to make human-friendly names using a class name as a template"""
    if name.endswith(suffix):
        name = name[: -len(suffix)]
    return camel_case_to_underscore(name)


def camel_case_to_underscore(name: str) -> str:
    """
    Convert name in camel-case notation into lowercase+underscore notation.

    For example, AbcXyz will be converted into abc_xyz.
    """
    s1 = re.sub(r"(.)([A-Z][a-z]+)", r"\1_\2", name)
    return re.sub(r"([a-z0-9])([A-Z])", r"\1_\2", s1).lower()


def normalize_string(s: str) -> str:
    """
    Strip parts of the raw string to try and make matching more effective.

    There are many redundant parts of input strings, or parts that don't
    meaningfully contribute to whether its a match or not. Try and strip
    as much of that as possible.
    """
    # Lowercase
    # CrAzY cAsE => crazy case
    s = s.lower()
    # Strip accent characters
    # ãóë => aoe
    s = "".join(
        c for c in unicodedata.normalize("NFKD", s) if not unicodedata.combining(c)
    )
    # Strip repeats of 2+
    # w0000000t => w00t
    s = re.sub("(.)(\1)+", "\1\1", s)
    # Strip non alphanumerics (including spaces)
    #
    s = re.sub(r"[\W_]", "", s)
    return s


def normalize_url(url: str) -> bytes:
    """
    Normalize the URL and strip the scheme from the URL to make matching more effective.

    Urls will be normalized to lowercase and the initial scheme removed as well as "utf-8" encoded.
    """
    # Lowercase
    # HtTPs://wWw.faCeBook.cOM => https://www.facebook.com
    url = url.lower()
    # parse the Url into it's consituent parts
    parsed = urlparse(url)
    # identify the scheme and trailing punctuation
    # e.g. scheme = "http://"
    scheme = "%s://" % parsed.scheme
    # Remove the scheme from the full url
    # https://www.facebook.com => www.facebook.com
    url = parsed.geturl().replace(scheme, "", 1)
    # Ensure URL is utf-8 encoded
    return url.encode("utf-8")


def argparse_choices_pre_type(choices: t.List[str], type: t.Callable[[str], t.Any]):
    """
    Argparse parses choices after type, which is sometimes undesirable.

    So fix it with duct tape. type=argparse_choices_pre_type()
    """

    def ret(s: str):
        if s not in choices:
            raise argparse.ArgumentTypeError(
                "invalid choice: {} (choose from {})".format(
                    s, ", ".join(repr(c) for c in choices)
                ),
            )
        return type(s)

    return ret


def argparse_choices_pre_type_kwargs(
    choices: t.List[str], type: t.Callable[[str], t.Any]
):
    """
    Argparse parses choices after type, which is sometimes undesirable.

    So fix it with duct tape. type=argparse_choices_pre_type()
    """

    def ret(s: str):
        if s not in choices:
            raise argparse.ArgumentTypeError(
                "invalid choice: {} (choose from {})".format(
                    s, ", ".join(repr(c) for c in choices)
                ),
            )
        return type(s)

    return {"type": ret, "metavar": "{%s}" % ",".join(choices)}



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved

import unittest
import typing as t

from threatexchange.signal_type.signal_base import TrivialSignalTypeIndex
from threatexchange.signal_type.index import SignalTypeIndex
from threatexchange.signal_type.pdq.pdq_index import PDQIndex


class TestIndexUpdates(unittest.TestCase):
    __test__ = False

    def get_first_set(self) -> t.Iterable[t.Tuple[str, t.Any]]:
        return []

    def get_second_set(self) -> t.Iterable[t.Tuple[str, t.Any]]:
        return []

    def get_index(
        self, initial_set: t.Iterable[t.Tuple[str, t.Any]]
    ) -> SignalTypeIndex:
        raise NotImplementedError

    def test_index_updates_actually_update_the_index(self):
        index = self.get_index(self.get_first_set())

        self.assertEqual(len(index.query(self.get_first_set()[0][0])), 1)
        self.assertEqual(len(index.query(self.get_second_set()[0][0])), 0)

        index.add_all(self.get_second_set())

        self.assertEqual(len(index.query(self.get_first_set()[0][0])), 1)
        self.assertEqual(len(index.query(self.get_second_set()[0][0])), 1)


class TestTrivialTypeIndexUpdates(TestIndexUpdates):
    __test__ = True

    def get_index(
        self, initial_set: t.Iterable[t.Tuple[str, t.Any]]
    ) -> SignalTypeIndex:
        return TrivialSignalTypeIndex.build(initial_set)

    def get_first_set(self) -> t.Iterable[t.Tuple[str, t.Any]]:
        return [
            ("55fc67a4e8d667d9668700c1920ff190", {"meta_data": 12}),
            ("5c4474f0e17bb56d0d73cca24c77e0d7", {"meta_data": 12}),
            ("e08572ca2db00e1f375d93b55b36a996", {"meta_data": 12}),
            ("ff2194f1208a9e07d41ad91ca64d3783", {"meta_data": 12}),
            ("07b261455844fcf348b56dc6511ffd72", {"meta_data": 12}),
            ("7f0ddfb32b98164a32bacc2bbc6f55ae", {"meta_data": 12}),
            ("36643e26bb7b12bc469fc9770c6e7252", {"meta_data": 12}),
            ("280791a55386db1997d4c9c21db8344b", {"meta_data": 12}),
        ]

    def get_second_set(self) -> t.Iterable[t.Tuple[str, t.Any]]:
        return [
            ("2624c75d21e5b991500812d84bf66d69", {"meta_data": 12}),
            ("46521c378a9070ce9da52f867c6eb024", {"meta_data": 12}),
            ("e757566a235fe9cef79478390c1a4d8b", {"meta_data": 12}),
            ("487a15edcec640fe8286b068e25ba2b3", {"meta_data": 12}),
            ("49442339bca61bda78a69a5de4d956f6", {"meta_data": 12}),
            ("5fd5ad963c42398ca8beabec76647d45", {"meta_data": 12}),
            ("353767ed43bfb3445086e16a0a7a6c52", {"meta_data": 12}),
            ("9c9657285b98b8cfb9a3de8570c6e8a1", {"meta_data": 12}),
            ("73a8d3f9fc517806673b50d965466943", {"meta_data": 12}),
            ("420e238441fb34901697f02f086ff466", {"meta_data": 12}),
        ]


class TestPdqIndexUpdates(TestIndexUpdates):
    __test__ = True

    def get_index(
        self, initial_set: t.Iterable[t.Tuple[str, t.Any]]
    ) -> SignalTypeIndex:
        return PDQIndex.build(initial_set)

    def get_first_set(self) -> t.Iterable[t.Tuple[str, t.Any]]:
        return [
            (
                "7c70f604bc24c9db8d9b5e3c7e7c3e3c1e3c0b3820db08e698e6c0c6c3c7e391",
                {"meta_data": 12},
            ),
            (
                "5e53f66078ec5b664fa338682f5866f96f404b4e671925e15e0e5d198663865c",
                {"meta_data": 12},
            ),
            (
                "2af1dc14d59f27eeada8dd54e05ca07b54881f8007f0e9f5a9fc265fdd02d180",
                {"meta_data": 12},
            ),
            (
                "d33c9979cbab36ba44ed72aa1358d6a7ac928d4168963512fbb511643336c4cb",
                {"meta_data": 12},
            ),
            (
                "dd24e2fc80f93cb6259c1f91b387e18fec71f03f38cefc07d40e3d88d2009f80",
                {"meta_data": 12},
            ),
            (
                "d63aa91c969b892494cacbb0d5ba994a6596d52a2db1d2b88daad20697b959f3",
                {"meta_data": 12},
            ),
            (
                "379dbbb1d27499aac9fa8c89bccc4e5b126ca5688ca572c119ce2c6418b3df1c",
                {"meta_data": 12},
            ),
            (
                "989038bb3c5b1a339d397f231f0b0f3f090caf768d5ecf27c685677a21309285",
                {"meta_data": 12},
            ),
            (
                "2f03781a0a98d2579160569fc7dcf97a2c80eda7192712de98e635fdd5651a16",
                {"meta_data": 12},
            ),
            (
                "5d871b14393d2533e6e3e6c6cecc98ceb989b93187b1f6346aa72a434a4bc898",
                {"meta_data": 12},
            ),
            (
                "600f3fa345ac8c55e39352c9ce2baf4828f4533610cadb93ae6d7d25d099cee2",
                {"meta_data": 12},
            ),
            (
                "3a50b8642da4569bd55bdaa9562d65ae5b132272f4e86b2a5949a575aaae5529",
                {"meta_data": 12},
            ),
            (
                "a6798c6971ce39ce6c319e70f0943d87387c0e03f078279ecf0f1f1c30df1863",
                {"meta_data": 12},
            ),
        ]

    def get_second_set(self) -> t.Iterable[t.Tuple[str, t.Any]]:
        return [
            (
                "002f5dd87fe4d01bff76203d803f07c517833ef907f2ae0f0044fc9c4013ff6a",
                {"meta_data": 12},
            ),
            (
                "5e74b3c76c1a48e4a7981f63c8cd3732878b5075a9b246459ae3f5fa4e24c9c1",
                {"meta_data": 12},
            ),
            (
                "8dff1f318060f01381c9e01fcee6d83ff831f189e00e9e047c76f381fdd007cb",
                {"meta_data": 12},
            ),
            (
                "0bd0b99c6374ac509981760ec9c3f4035e8b303e44eb55ab03fa00fec1faffd5",
                {"meta_data": 12},
            ),
            (
                "2d24e51c3c76245c9bf31d20239555c88a77b496ae53e412baf7aab352d34b4e",
                {"meta_data": 12},
            ),
            (
                "454db29134aa3b565f78f9abb1c51ce58c2a2a7485724163fc39af98569ce81e",
                {"meta_data": 12},
            ),
            (
                "b0f815db7693c05a4b765a66584dd92cdb952c31b46335e124ce049c7eb82ddb",
                {"meta_data": 12},
            ),
            (
                "d864307439ed1784e39ce2192c7a1e7a77e3c1e38987199d9bbcfeb804426dc2",
                {"meta_data": 12},
            ),
            (
                "378948acdbf83661247ebbdfcc39863631af7d0acb54249c0501cfb6fc127823",
                {"meta_data": 12},
            ),
            (
                "c4b54f9e32c67134c704ce16dc6d31873634b339c263c44cc9c7958f1a396fb5",
                {"meta_data": 12},
            ),
            (
                "5a6a69bd8016d6c2c37a6ca5d38ca83f97c24f2830b595a5eeb56231f88df225",
                {"meta_data": 12},
            ),
            (
                "7ef85906403879e4a9f03c5ecf71285e4f7b9f40be9b010efd8001cea3c62f78",
                {"meta_data": 12},
            ),
            (
                "361da9e6cf1b72f5cea0344e5bb6e70939f4c70328ace762529cac704297354a",
                {"meta_data": 12},
            ),
        ]



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved

from threatexchange.extensions.manifest import ThreatExchangeExtensionManifest
from threatexchange.extensions.tlsh.text_tlsh import TextTLSHSignal


TX_MANIFEST = ThreatExchangeExtensionManifest(
    signal_types=(TextTLSHSignal,),
)



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved

import unittest

import threatexchange.common


class TestCommon(unittest.TestCase):
    def test_camel_case_to_underscore(self):
        assert threatexchange.common.camel_case_to_underscore("AbcXyz") == "abc_xyz"



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved

"""
CLI exceptions. Alone to prevent circular imports.
"""

from enum import IntEnum


class ReturnCode(IntEnum):
    """Check out http://tldp.org/LDP/abs/html/exitcodes.html"""

    SUCCESS = 0
    GENERAL_ERROR = 1
    USER_ERROR = 2
    EXTERNAL_DEPENDENCY_ERROR = 3

    INTERRUPT = 130
    MAX = 255


class CommandError(Exception):
    """Wrapper for exceptions which cause return codes"""

    def __init__(
        self, message: str, returncode: int = ReturnCode.GENERAL_ERROR.value
    ) -> None:
        super().__init__(message)
        self.returncode = returncode

    @classmethod
    def user(cls, message: str) -> "CommandError":
        """When PEBKAC"""
        return cls(message, 2)

    @classmethod
    def external_dependency(cls, message: str) -> "CommandError":
        """When you don't have internet access or the APIs are down"""
        return cls(message, ReturnCode.EXTERNAL_DEPENDENCY_ERROR.value)



#!/usr/bin/env python
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved

"""
Util file for Optical character recognition (OCR) related functions.
Use of pytesseract requires additional libaries already be installed, see https://github.com/madmaze/pytesseract#installation
"""

import pathlib
import pytesseract
import warnings
from PIL import Image


def text_from_image_file(path: pathlib.Path):
    """
    Given a path to a file return predicted OCR text
    Current tested against: jpg
    """
    img_pil = Image.open(path)
    try:
        return pytesseract.image_to_string(img_pil)
    except pytesseract.TesseractNotFoundError as e:
        warnings.warn(
            str(e),
            category=UserWarning,
        )
    return ""



from unittest.mock import Mock
import urllib.parse
import typing as t
import pytest
import requests
from threatexchange.exchanges.clients.ncmec.hash_api import (
    NCMECEntryType,
    NCMECEntryUpdate,
    NCMECHashAPI,
    NCMECEnvironment,
)

STATUS_XML = """
<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>
<status xmlns="https://hashsharing.ncmec.org/hashsharing/v2">
    <ipAddress>127.0.0.1</ipAddress>
    <username>testington</username>
    <member id="1">Sir Testington</member>
</status>
""".strip()

NEXT_UNESCAPED = (
    "/v2/entries?from=2017-10-20T00%3A00%3A00.000Z"
    "&to=2017-10-30T00%3A00%3A00.000Z&start=2001&size=1000&max=3000"
)

NEXT_UNESCAPED2 = (
    "/v2/entries?from=2017-10-20T00%3A00%3A00.000Z"
    "&to=2017-10-30T00%3A00%3A00.000Z&start=3001&size=1000&max=4000"
)
NEXT_UNESCAPED3 = (
    "/v2/entries?from=2017-10-20T00%3A00%3A00.000Z"
    "&to=2017-10-30T00%3A00%3A00.000Z&start=4001&size=1000&max=5000"
)

ENTRIES_XML = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<queryResult xmlns="https://hashsharing.ncmec.org/hashsharing/v2">
    <images count="2" maxTimestamp="2017-10-24T15:10:00Z">
        <image>
            <member id="42">Example Member</member>
            <timestamp>2017-10-24T15:00:00Z</timestamp>
            <id>image1</id>
            <classification>A1</classification>
            <fingerprints>
                <md5>a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1</md5>
                <sha1>a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1</sha1>
                <pdna>a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1...</pdna>
            </fingerprints>
        </image>
        <deletedImage>
            <member id="43">Example Member2</member>
            <id>image4</id>
            <timestamp>2017-10-24T15:10:00Z</timestamp>
        </deletedImage>
    </images>
    <videos count="2" maxTimestamp="2017-10-24T15:20:00Z">
        <video>
            <member id="42">Example Member</member>
            <timestamp>2017-10-24T15:00:00Z</timestamp>
            <id>video1</id>
            <fingerprints>
                <md5>b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1</md5>
                <sha1>b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1</sha1>
            </fingerprints>
        </video>
        <deletedVideo>
            <member id="42">Example Member</member>
            <id>video4</id>
            <timestamp>2017-10-24T15:20:00Z</timestamp>
        </deletedVideo>
    </videos>
    <paging>
        <next>/v2/entries?from=2017-10-20T00%3A00%3A00.000Z&amp;to=2017-10-30T00%3A00%3A00.000Z&amp;start=2001&amp;size=1000&amp;max=3000</next>
    </paging>
</queryResult>
""".strip()


ENTRIES_XML2 = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<queryResult xmlns="https://hashsharing.ncmec.org/hashsharing/v2">
    <images count="1" maxTimestamp="2019-10-24T15:10:00Z">
        <image>
            <member id="42">Example Member</member>
            <timestamp>2019-10-24T15:00:00Z</timestamp>
            <id>image10</id>
            <classification>A1</classification>
            <fingerprints>
                <md5>b1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1</md5>
                <sha1>b1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1</sha1>
                <pdna>b1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1...</pdna>
            </fingerprints>
        </image>
    </images>
    <paging>
        <next>/v2/entries?from=2017-10-20T00%3A00%3A00.000Z&amp;to=2017-10-30T00%3A00%3A00.000Z&amp;start=3001&amp;size=1000&amp;max=4000</next>
    </paging>
</queryResult>
""".strip()

# This example isn't in the documentation, but shows how updates work
ENTRIES_XML3 = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<queryResult xmlns="https://hashsharing.ncmec.org/hashsharing/v2">
    <videos count="2" maxTimestamp="2019-11-25T15:10:00Z">
        <video>
            <member id="101">TX Example</member>
            <timestamp>2019-11-25T15:10:00Z</timestamp>
            <id>willupdate</id>
            <classification>A1</classification>
            <fingerprints>
                <md5>facefacefacefacefacefacefaceface</md5>
            </fingerprints>
        </video>
        <video>
            <member id="101">TX Example</member>
            <timestamp>2019-11-24T15:10:00Z</timestamp>
            <id>willdelete</id>
            <classification>A1</classification>
            <fingerprints>
                <md5>bacebacebacebacebacebacebacebace</md5>
            </fingerprints>
        </video>
    </videos>
    <paging>
        <next>/v2/entries?from=2017-10-20T00%3A00%3A00.000Z&amp;to=2017-10-30T00%3A00%3A00.000Z&amp;start=4001&amp;size=1000&amp;max=5000</next>
    </paging>
</queryResult>
""".strip()

ENTRIES_XML4 = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<queryResult xmlns="https://hashsharing.ncmec.org/hashsharing/v2">
    <videos count="2" maxTimestamp="2019-11-24T15:10:00Z">
        <video>
            <member id="101">TX Example</member>
            <timestamp>2019-11-24T15:10:00Z</timestamp>
            <id>willupdate</id>
            <classification>A2</classification>
            <fingerprints>
                <md5>facefacefacefacefacefacefaceface</md5>
            </fingerprints>
        </video>
        <deletedVideo>
            <member id="101">TX Example</member>
            <timestamp>2019-11-25T15:10:00Z</timestamp>
            <id>willdelete</id>
        </deletedVideo>
    </videos>
</queryResult>
""".strip()

ENTRIES_LARGE_FINGERPRINTS = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<queryResult xmlns="https://hashsharing.ncmec.org/hashsharing/v2">
    <videos count="1" maxTimestamp="2019-11-24T15:10:00Z">
        <video>
            <member id="101">TX Example</member>
            <timestamp>2019-11-24T15:10:00Z</timestamp>
            <id>largetags</id>
            <classification>A2</classification>
            <fingerprints>
                <md5>facefacefacefacefacefacefaceface</md5>
                <tmk-pdqf rel="self" href="/v2/entries/1/fingerprints/TMK_PDQF"/>
				<videntifier rel="self" href="/v2/entries/1/fingerprints/VIDENTIFIER"/>
            </fingerprints>
        </video>
    </videos>
</queryResult>
""".strip()


def mock_get_impl(url: str, **params):
    content = ENTRIES_XML
    if url.endswith(NEXT_UNESCAPED):
        content = ENTRIES_XML2
    if url.endswith(NEXT_UNESCAPED2):
        content = ENTRIES_XML3
    if url.endswith(NEXT_UNESCAPED3):
        content = ENTRIES_XML4
    # Void your warantee by messing with requests state
    resp = requests.Response()
    resp._content = content.encode()
    resp.status_code = 200
    resp.content  # Set the rest of Request's internal state
    return resp


def set_api_return(content: str):
    # Some day support next
    # def next_str(i: int) -> str:
    #     return (
    #         "/v2/entries?from=2017-10-20T00%3A00%3A00.000Z"
    #         f"&to=2017-10-30T00%3A00%3A00.000Z&start={i * 1000 + 1}&size=1000&max={(i + 1) * 1000}"
    #     )
    #
    # content = xmls[(int(qs.get("start", "1")) - 1) // 1000]

    def _mock_get_impl(url: str, **params):
        resp = requests.Response()
        resp._content = content.encode()
        resp.status_code = 200
        resp.content  # Set the rest of Request's internal state
        return resp

    return _mock_get_impl


@pytest.fixture
def api(monkeypatch: pytest.MonkeyPatch):
    api = NCMECHashAPI("fake_user", "fake_pass", NCMECEnvironment.test_Industry)
    session = None
    session = Mock(
        strict_spec=["get", "__enter__", "__exit__"],
        get=mock_get_impl,
        __enter__=lambda _: session,
        __exit__=lambda *args: None,
    )
    monkeypatch.setattr(api, "_get_session", lambda: session)
    return api


def assert_first_entry(entry: NCMECEntryUpdate) -> None:
    assert entry.id == "image1"
    assert entry.member_id == 42
    assert entry.entry_type == NCMECEntryType.image
    assert entry.deleted is False
    assert entry.fingerprints == {
        "md5": "a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1",
        "sha1": "a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1",
        "pdna": "a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1...",
    }


def assert_second_entry(entry: NCMECEntryUpdate) -> None:
    assert entry.id == "image4"
    assert entry.member_id == 43
    assert entry.entry_type == NCMECEntryType.image
    assert entry.deleted is True
    assert entry.fingerprints == {}


def assert_third_entry(entry: NCMECEntryUpdate) -> None:
    assert entry.id == "video1"
    assert entry.member_id == 42
    assert entry.entry_type == NCMECEntryType.video
    assert entry.deleted is False
    assert entry.fingerprints == {
        "md5": "b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1",
        "sha1": "b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1b1",
    }


def assert_fourth_entry(entry: NCMECEntryUpdate) -> None:
    assert entry.id == "video4"
    assert entry.member_id == 42
    assert entry.entry_type == NCMECEntryType.video
    assert entry.deleted is True
    assert entry.fingerprints == {}


def assert_fifth_entry(entry: NCMECEntryUpdate) -> None:
    assert entry.id == "image10"
    assert entry.member_id == 42
    assert entry.entry_type == NCMECEntryType.image
    assert entry.deleted is False
    assert entry.fingerprints == {
        "md5": "b1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1",
        "sha1": "b1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1",
        "pdna": "b1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1a1...",
    }


def test_mocked_get_hashes(api: NCMECHashAPI):
    result = api.get_entries()

    assert len(result.updates) == 4
    assert result.max_timestamp == 1508858400
    assert result.next != ""
    one, two, three, four = result.updates
    assert_first_entry(one)
    assert_second_entry(two)
    assert_third_entry(three)
    assert_fourth_entry(four)

    second_result = api.get_entries(next_=result.next)

    assert len(second_result.updates) == 1
    assert second_result.max_timestamp == 1571929800
    assert second_result.next != ""
    five = second_result.updates[0]
    assert_fifth_entry(five)

    # These later results don't need to be tested for this test, matters
    # for the SignalExchange API
    third_result = api.get_entries(next_=second_result.next)
    assert third_result.next != ""
    assert third_result not in (result, second_result)
    forth_result = api.get_entries(next_=third_result.next)
    assert forth_result not in (result, second_result, third_result)
    assert forth_result.next == ""

    # The other updates don't need to be tested here


def test_large_fingerprint_entries(monkeypatch):
    api = NCMECHashAPI("fake_user", "fake_pass", NCMECEnvironment.test_Industry)
    session = Mock(
        strict_spec=["get", "__enter__", "__exit__"],
        get=set_api_return(ENTRIES_LARGE_FINGERPRINTS),
        __enter__=lambda _: session,
        __exit__=lambda *args: None,
    )
    monkeypatch.setattr(api, "_get_session", lambda: session)

    result = api.get_entries()

    assert len(result.updates) == 1
    update = result.updates[0]
    assert len(update.fingerprints) == 1
    assert update.fingerprints == {"md5": "facefacefacefacefacefacefaceface"}
    assert result.next == ""
